function [offset] = mtb(imgArray, max_offset) 
%mtb Calculate offsets for imgArray through Medium Threshold Bitmap.
%   Calculate the offsets for all the images in the imgArray. The images
%   will be aligned after the first image. The max_offset gives how many
%   steps the pyramid will have by log2. In this range will be looked for
%   alignment. Good values are 64-128.

% generate mtb images
[x_res, y_res, n] = size(imgArray);
thresh            = floor(log2(max_offset));
mtbCell           = cell(thresh, n);
for i=1:n
    % calculate first mtb
    tmp = double(imgArray(:, :, i));
    med = median(tmp(:));
    mtbCell(1, i) = {imgArray(:, :, i) >= med};
    
    % generate pyramid
    for j=2:thresh
        prev_mtb      = mtbCell{j-1, i};
        mtbCell(j, i) = {impyramid(prev_mtb(:, :), 'reduce')};
    end
end

% now we have:
% - images in a cell array imgCell
% - tresh mtbs in mtbCell
% - each mtbCell contains a 3D matrix (:,:,k) with k giving the image

% generate the offset for alignment for first image in cell as reference
offset = zeros(2, n);
for i=2:n
    best_offset = zeros(2, 1);
    for j=thresh:-1:1
        % increase offset by resolution
        best_offset    = best_offset*2;
        offset(:, i)   = best_offset;
        best_error     = Inf;            
        original       = mtbCell{j, 1};
        toadjust       = mtbCell{j, i};
        [x_res, y_res] = size(original);
        overlap        = zeros(2,4);
        
        % compare offsets in a range of -1,+1 for each dimension
        for x=-1:1
            for y=-1:1
                x_offset = offset(1, i) + x;
                y_offset = offset(2, i) + y;
                
                % calculate overlap, original always from 1 to x/yres             
                % original on top of toadjust?
                if 1 < (1+x_offset)
                    overlap(1, 1:4) = [1+x_offset 1+x_offset x_res x_res];
                else
                    overlap(1, 1:4) = [1 1 x_res+x_offset x_res+x_offset];
                end
                % original left of toadjust?
                if 1 < (1+y_offset)
                    overlap(2, 1:4) = [1+y_offset y_res 1+y_offset y_res];                 
                else
                    overlap(2, 1:4) = [1 y_res+y_offset 1 y_res+y_offset];
                end
                
                % sample overlap
                x_from = overlap(1,1);
                x_to   = overlap(1,3);
                y_from = overlap(2,1);
                y_to   = overlap(2,2);
                or_sample = original(x_from:x_to, y_from:y_to);
                x_from = x_from - x_offset;
                x_to   = x_to   - x_offset;
                y_from = y_from - y_offset;
                y_to   = y_to   - y_offset;
                ta_sample = toadjust(x_from:x_to, y_from:y_to);
                
                % calculate missalignment
                dif       = xor(or_sample, ta_sample);
                new_error = sum(dif(:));
                
                % check if error is minimized
                if new_error < best_error
                    best_offset = [x_offset y_offset];
                    best_error  = new_error;
                end
            end
        end
        offset(:,i) = best_offset;
    end
end